@@ -38,10 +38,13 @@
 
 #include <cstdio>
 #include <memory>
+#include <sstream>
 
 #ifndef STANDALONE
-#include <objtools/readers/seqdb/seqdb.hpp>
-#include <objtools/readers/seqdb/seqdbcommon.hpp>
+//#include <objtools/readers/seqdb/seqdb.hpp>
+//#include <objtools/readers/seqdb/seqdbcommon.hpp>
+#include <objtools/blast/seqdb_reader/seqdb.hpp>
+#include <objtools/blast/seqdb_reader/seqdbcommon.hpp>
 #endif
 
 USING_NCBI_SCOPE;
@@ -374,7 +377,7 @@ public:
             if(pos != m_SeqLen) {
                 if(m_Flags&fSequenceProgress ) {
                     if(m_Flags&fSequencePosition) fprintf(stderr,"%u/%u ",pos,m_SeqLen);
-                    if(m_Flags&fSequencePercent) fprintf(stderr,"%5.1lf% ",(pos*100.0)/m_SeqLen);
+                    if(m_Flags&fSequencePercent) fprintf(stderr,"%5.1lf%% ",(pos*100.0)/m_SeqLen);
                     if(m_Flags&fNewlineAtProgress) putc('\n',stderr);
                 }
             } else {
@@ -692,23 +695,51 @@ int CMain::Execute()
 				f!=fafiles.end(); ++f) {
 #ifndef STANDALONE
 				if( m_blastdbs ) {
-					CSeqDBGiList * lst = (m_gilist.length() ? new CSeqDBFileGiList( m_gilist ) : 0 );
-					CSeqDB seqDB( *f, CSeqDB::eNucleotide, lst );
-					processor.CbkFileBegin();
-					for( CSeqDBIter i = seqDB.Begin(); i; ++i ) {
-						list<CRef<CSeq_id> > ids = seqDB.GetSeqIDs( i.GetOID() );
-						string ident;
-						for( list<CRef<CSeq_id> >::const_iterator x = ids.begin(); x != ids.end(); ++x ) {
-							if( x != ids.begin() ) ident += "|";
-							ident += (*x)->AsFastaString();
+                    vector<string> volumes;
+                    try {
+                        CSeqDB::FindVolumePaths( *f, CSeqDB::eNucleotide, volumes, 0, true );
+                    } catch(exception& e) {
+                        cerr << "? Warning: CSeqDB::FindVolumePaths( \"" << *f << "\", CSeqDB::eNucleotide, volumes, 0, true ); failed with error: " << e.what() << "\n";
+                        volumes.clear();
+                        volumes.push_back( *f );
+                    } catch(...) {
+                        cerr << "? Warning: CSeqDB::FindVolumePaths( \"" << *f << "\", CSeqDB::eNucleotide, volumes, 0, true ); failed with unknown exception\n";
+                        volumes.clear();
+                        volumes.push_back( *f );
+                    }
+                    for( vector<string>::const_iterator v = volumes.begin(); v != volumes.end(); ++v ) {
+						auto_ptr<CSeqDB> seqDB( 0 );
+                        try {
+					        CSeqDBGiList * lst = (m_gilist.length() ? new CSeqDBFileGiList( m_gilist ) : 0 );
+                            seqDB.reset( new CSeqDB( *v, CSeqDB::eNucleotide, lst ) );
+                        } catch(exception& e) {
+                            throw runtime_error( "Failed to open blastdb volume " + *v + ": " + e.what() );
+                        } catch(...) {
+                            throw runtime_error( "Failed to open blastdb volume " + *v + ": unknown error" );
+                        }
+						processor.CbkFileBegin();
+						for( CSeqDBIter i = seqDB->Begin(); i; ++i ) {
+							list<CRef<CSeq_id> > ids = seqDB->GetSeqIDs( i.GetOID() );
+                            if( ids.size() == 0 ) {
+                                ostringstream err;
+                                err << "Bad entry in " << *f << " (" << *v << ") " << " oid " << i.GetOID() << ": no seqids\n";
+                                throw runtime_error( err.str() );
+                                //cerr << "? Warning: " << err.str();
+                                //continue;
+                            }
+							string ident;
+							for( list<CRef<CSeq_id> >::const_iterator x = ids.begin(); x != ids.end(); ++x ) {
+								if( x != ids.begin() ) ident += "|";
+								ident += (*x)->AsFastaString();
+							}
+							processor.CbkEntryBegin();
+							processor.CbkIdent( ident.c_str(), ident.length() );
+							string seq;
+							seqDB->GetSequenceAsString( i.GetOID(), seq );
+							processor.CbkSeqline( seq.c_str(), seq.length() );
+							processor.CbkEntryEnd();
 						}
-						processor.CbkEntryBegin();
-						processor.CbkIdent( ident.c_str(), ident.length() );
-						string seq;
-						seqDB.GetSequenceAsString( i.GetOID(), seq );
-						processor.CbkSeqline( seq.c_str(), seq.length() );
-						processor.CbkEntryEnd();
-					}
+                    }
 					processor.CbkFileEnd();
 				} else {
 #endif
@@ -757,9 +788,10 @@ int CMain::Run()
 
 int main(int argc, char ** argv) 
 {
-	try {
+//	try {
 		CMain app(argc,argv);
 		return app.Run();
+        /*
 	}
 	catch(logic_error& e) {
 		fprintf(stderr,"! Fatal: Internal error %s\n",e.what());
@@ -771,6 +803,7 @@ int main(int argc, char ** argv)
 		fprintf(stderr,"! Fatal: Unknown error\n");
 	}
 	return 100;
+    */
 }
 
 
