@@ -183,7 +183,7 @@ void CFaLookup::AttachFile( const string& path )
 // 			last_off,
 // 			tail_off);
 	
-	if( last_off != tail_off + 3 * sizeof( off64_t ) ) {
+	if( last_off != off64_t(tail_off + 3 * sizeof( off64_t )) ) {
 		throw runtime_error( "Wrong file trailer in " + path );
 	}
 	
